@@ -84,6 +84,47 @@
     }
   }
   
+  if (solver == "highs") {
+    #library(highs)
+    lhs = rep(-Inf, length(sense))
+    rhs = rep(Inf, length(sense))
+    lhs[sense == "G"] = bvec[sense == "G"]
+    rhs[sense == "L"] = bvec[sense == "L"]
+    lhs[sense == "E"] = bvec[sense == "E"]
+    rhs[sense == "E"] = bvec[sense == "E"]
+    
+    types = vtype
+    types[types=="B"] = "I"
+    
+    cat(format("  Finding the optimal matches..."), "\n")
+    ptm = proc.time()
+    out = highs_solve(L = cvec,
+                      lower = 0,
+                      upper = ub,
+                      A = Amat,
+                      lhs = lhs,
+                      rhs = rhs,
+                      types = types,
+                      maximum = TRUE)
+    time = (proc.time()-ptm)[3]
+    if (out$status == 8) {
+      cat(format("  Error: problem infeasible!"), "\n")
+      obj = 0
+      sol = NULL
+    }
+    
+    else if (out$status == 7 | out$status == 13){
+      if (out$status == 7){
+        cat(format("  Optimal matches found"), "\n")
+      }
+      else if (out$status == 13){
+        cat(format("  Time limit reached!"), "\n")
+      }
+      sol = out$primal_solution
+      obj = sum((t(dist_mat)[lower.tri(dist_mat)]-(subset_weight*rep(1, n_dec))) * (round(out$primal_solution, 1e-10) == 1))
+    }
+  }
+  
   if (solver == "symphony") {
     #library(Rsymphony)
     if (requireNamespace('Rsymphony', quietly = TRUE)) {
@@ -115,46 +156,51 @@
   # GLPK
   if (solver == "glpk") {
     #library(Rglpk)
-    #dir = rep(NA, length(sense))
-    #dir[sense=="E"] = '=='
-    #dir[sense=="L"] = '<='
-    #dir[sense=="G"] = '>='
-    #
-    #bound = list(lower = list(ind=c(1:length(ub)), val=rep(0,length(ub))),
-    #             upper = list(ind=c(1:length(ub)), val=ub))
-    #
-    #ptm = proc.time()
-    #out= Rglpk_solve_LP(cvec, Amat, dir, bvec, bounds = bound, types = vtype, max = TRUE)
-    #time = (proc.time()-ptm)[3]
-    #
-    #if (out$status!=0) {
-    #  cat(format("  Error: problem infeasible!"), "\n")
-    #  obj = 0
-    #  sol = NULL
-    #} else {
-    #  sol = out$solution
-    #  obj = sum((t(dist_mat)[lower.tri(dist_mat)]-(subset_weight*rep(1, n_dec))) * out$solution)
-    #}
-    ptm = proc.time()
-    Amat = matrix(0, nrow = n_tot, ncol = n_tot)
-    res = matrix(0, nrow = n_tot, ncol = n_tot)
-    
-    Amat[upper.tri(Amat)] = coef
-    
-    max_edge = max(Amat)
-    while (max_edge > 0) {
-      row = which(Amat == max_edge, arr.ind=TRUE)[1,1]
-      col = which(Amat == max_edge, arr.ind=TRUE)[1,2]
-      res[row,col] = 1
-      Amat[row,] = 0
-      Amat[,row] = 0
-      Amat[col,] = 0
-      Amat[,col] = 0
+    if (requireNamespace('Rglpk', quietly = TRUE)) {
+      #dir = rep(NA, length(sense))
+      #dir[sense=="E"] = '=='
+      #dir[sense=="L"] = '<='
+      #dir[sense=="G"] = '>='
+      #
+      #bound = list(lower = list(ind=c(1:length(ub)), val=rep(0,length(ub))),
+      #             upper = list(ind=c(1:length(ub)), val=ub))
+      #
+      #ptm = proc.time()
+      #out= Rglpk::Rglpk_solve_LP(cvec, Amat, dir, bvec, bounds = bound, types = vtype, max = TRUE)
+      #time = (proc.time()-ptm)[3]
+      #
+      #if (out$status!=0) {
+      #  cat(format("  Error: problem infeasible!"), "\n")
+      #  obj = 0
+      #  sol = NULL
+      #} else {
+      #  sol = out$solution
+      #  obj = sum((t(dist_mat)[lower.tri(dist_mat)]-(subset_weight*rep(1, n_dec))) * out$solution)
+      #}
+      ptm = proc.time()
+      Amat = matrix(0, nrow = n_tot, ncol = n_tot)
+      res = matrix(0, nrow = n_tot, ncol = n_tot)
+      
+      Amat[upper.tri(Amat)] = coef
+      
       max_edge = max(Amat)
+      while (max_edge > 0) {
+        row = which(Amat == max_edge, arr.ind=TRUE)[1,1]
+        col = which(Amat == max_edge, arr.ind=TRUE)[1,2]
+        res[row,col] = 1
+        Amat[row,] = 0
+        Amat[,row] = 0
+        Amat[col,] = 0
+        Amat[,col] = 0
+        max_edge = max(Amat)
+      }
+      sol = res[upper.tri(res)]
+      obj = sum((t(dist_mat)[lower.tri(dist_mat)]-(subset_weight*rep(1, n_dec))) * sol)
+      time = (proc.time()-ptm)[3]
+    }
+    else {
+      stop('suggested package not installed')
     }
-    sol = res[upper.tri(res)]
-    obj = sum((t(dist_mat)[lower.tri(dist_mat)]-(subset_weight*rep(1, n_dec))) * sol)
-    time = (proc.time()-ptm)[3]
   }
   
   
